package org.openmole.gate.server

import com.mongodb.{MongoCredential, ServerAddress}
import com.mongodb.casbah.Imports.{MongoCollection, MongoDBObject}
import com.mongodb.casbah.MongoClient
import org.openmole.gate.server.Data.User
import org.scalatra._
import org.scalatra.auth.{ScentryConfig, ScentrySupport}
import org.scalatra.auth.strategy.{BasicAuthStrategy, BasicAuthSupport}
import server.DBStrategy
import server.Launcher.MongoDB

import scala.concurrent.duration._
import scala.concurrent.Await
import scalatags.Text.all._
import scalatags.Text.{all => tags}
import scala.concurrent.ExecutionContext.Implicits.global
import shared._

object AutowireServer extends autowire.Server[String, upickle.default.Reader, upickle.default.Writer] {
  def read[Result: upickle.default.Reader](p: String) = upickle.default.read[Result](p)
  def write[Result: upickle.default.Writer](r: Result) = upickle.default.write(r)
}

class Servlet(mongoDB: MongoDB) extends ScalatraServlet with AuthenticationSupport {

  val basePath = "shared"

  val head = tags.head(
    tags.meta(tags.httpEquiv := "Content-Type", tags.content := "text/html; charset=UTF-8"),
    tags.link(tags.rel := "stylesheet", tags.`type` := "text/css", href := "css/bootstrap.min.css"),
    tags.script(tags.`type` := "text/javascript", tags.src := "js/gate.js")
  )

  get("/") {
    contentType = "text/html"
    auth("DB") match {
      case Some(user) =>
        val userSession = MongoDBObject(user.login -> session.getId)
        println(mongoDB.host + " => " + mongoDB.port)
        val server = new ServerAddress(mongoDB.host, mongoDB.port)
        val credentials = MongoCredential.createCredential("accountUser", "products", "password".toCharArray)
        val mongoClient = MongoClient(server, List(credentials))
        println(mongoDB.db + " ? " + mongoDB.dataSet)
        val mongoColl = mongoClient(mongoDB.db)(mongoDB.dataSet)
        mongoColl.findOne(userSession) match {
          case Some(x) =>
            tags.html(
              head,
              tags.body(
                div("already authenticated: " + user.login),
                script("client.Gate.run();")
              )
            )
          case None =>
            mongoColl += userSession
            tags.html(
              head,
              tags.body(
                div("authenticated: " + user.login),
                script("client.Gate.run();")
              )
            )
        }
      case None =>
        tags.html(
          head,
          tags.body(
            div("Failed to authenticate")
          )
        )
    }
  }

  post(s"/$basePath/*") {
    Await.result(AutowireServer.route[Api](ApiImpl)(
      autowire.Core.Request(Seq(basePath) ++ multiParams("splat").head.split("/"),
        upickle.default.read[Map[String, String]](request.body))
    ), Duration.Inf)
  }

}

object Data {
  type Login = String
  type Password = String

  case class User(login: Login)
}

trait AuthenticationSupport extends ScentrySupport[Data.User] with BasicAuthSupport[Data.User] {
  this: Servlet ⇒

  val realm = "Gate authentication"

  protected def fromSession = {
    case id: String ⇒ Data.User(id)
  }

  protected def toSession = {
    case usr: Data.User ⇒ usr.login
  }

  protected val scentryConfig = (new ScentryConfig {}).asInstanceOf[ScentryConfiguration]

  override protected def configureScentry = {
    scentry.unauthenticated {
      scentry.strategies("DB").unauthenticated()
    }
  }

  override protected def registerAuthStrategies = {
    scentry.register("DB", app ⇒ new DBStrategy(app, realm))
  }

  def auth(sch:String) = {
    val baReq = new BasicAuthStrategy.BasicAuthRequest(request)
    if (!baReq.providesAuth) {
      response.setHeader("WWW-Authenticate", "Basic realm=\"%s\"" format realm)
      halt(401, "Unauthenticated")
    }
    if (!baReq.isBasicAuth) {
      halt(400, "Bad Request")
    }

    scentry.authenticate(sch)
  }
}
